use std::fmt::Debug;

use binrw::{BinRead, BinWrite};
use diff::Diff;

use crate::{
    field_of::FieldOf,
    types::{Coordinate, ObjectType, F64, U32},
};

use super::{ObjectCore, ObjectModifier, Translate};

#[derive(BinRead, BinWrite, Clone, Debug, Diff, PartialEq)]
#[diff(attr(
    #[derive(Debug, PartialEq)]
))]
pub struct Polygon {
    pub core: ObjectCore,
    #[brw(magic(10u32))] // Number of following fields in struct
    pub invert_shape: U32,
    pub drawn_corner_a: FieldOf<Coordinate>,
    pub drawn_corner_b: FieldOf<Coordinate>,
    pub offset_cx: F64,
    pub offset_cy: F64,
    pub offset_dx: F64,
    pub offset_dy: F64,
    pub edges: U32,
    pub modifier: ObjectModifier,
}

impl Default for Polygon {
    fn default() -> Self {
        Self {
            core: ObjectCore::default(ObjectType::Polygon),
            invert_shape: 1.into(),
            drawn_corner_a: Coordinate { x: 0.0, y: 0.0 }.into(),
            drawn_corner_b: Coordinate { x: 0.0, y: 0.0 }.into(),
            offset_cx: 0.0.into(),
            offset_cy: 0.0.into(),
            offset_dx: 0.0.into(),
            offset_dy: 0.0.into(),
            edges: 6.into(),
            modifier: ObjectModifier::default(),
        }
    }
}

impl Translate for Polygon {
    fn move_absolute(&mut self, origin: Option<Coordinate>, z: Option<f64>) {
        origin.map(|origin| {
            let delta: Coordinate = origin - *self.core.origin;
            self.modifier.move_relative(delta);
        });
        self.core.move_absolute(origin, z);
    }

    fn move_relative(&mut self, delta: Option<Coordinate>, z: Option<f64>) {
        delta.map(|delta| {
            self.modifier.move_relative(delta);
        });
        self.core.move_relative(delta, z);
    }
}
